@@ -33,7 +33,7 @@ def pop_arg(arg, args, default=None):
     parser.add_argument(arg)
     namespace, args = parser.parse_known_args(args)
     namespace = tuple(vars(namespace).items())
-    if len(namespace) > 0 and namespace[0][1] is not None:
+    if namespace and namespace[0][1] is not None:
         val = namespace[0][1]
     else:
         val = default
@@ -55,21 +55,70 @@ def _remove_cwd_prefix(path):
     return result
 
 
+def has_cmake_cache_arg(cmake_args, arg_name, arg_value=None):
+    """Return True if ``-D<arg_name>:TYPE=<arg_value>`` is found
+    in ``cmake_args``. If ``arg_value`` is None, return True only if
+    ``-D<arg_name>:`` is found in the list."""
+    for arg in cmake_args:
+        if arg.startswith("-D%s:" % arg_name):
+            if arg_value is None:
+                return True
+            elif "=" in arg:
+                return arg.split("=")[1] == arg_value
+    return False
+
+
+def get_cmake_version(cmake_executable='cmake'):
+    """Runs CMake and extracts associated version information.
+    Raises :class:`skbuild.exceptions.SKBuildError` if it failed to execute CMake.
+    """
+    try:
+        version_string = subprocess.check_output([cmake_executable, '--version'])
+    except (OSError, subprocess.CalledProcessError):
+        raise SKBuildError(
+            "Problem with the CMake installation, aborting build. CMake executable is %s" % cmake_executable)
+
+    if sys.version_info > (3, 0):
+        version_string = version_string.decode()
+
+    return version_string.splitlines()[0].split(' ')[-1]
+
+
 class CMaker(object):
     """Interface to CMake executable."""
 
-    def __init__(self):
-        # verify that CMake is installed
-        try:
-            subprocess.check_call(['cmake', '--version'])
-        except (OSError, subprocess.CalledProcessError):
-            raise SKBuildError(
-                "Problem with the CMake installation, aborting build.")
-
+    def __init__(self, cmake_executable='cmake'):
+        self.cmake_executable = cmake_executable
+        self.cmake_version = get_cmake_version(self.cmake_executable)
         self.platform = get_platform()
 
+    def get_cached_generator_name(self):
+        """Reads and returns the cached generator from the :const:`skbuild.constants.CMAKE_BUILD_DIR`:.
+        Returns None if not found.
+        """
+        try:
+            cmake_generator = 'CMAKE_GENERATOR:INTERNAL='
+            with open(os.path.join(CMAKE_BUILD_DIR, 'CMakeCache.txt')) as fp:
+                for line in fp:
+                    if line.startswith(cmake_generator):
+                        return line[len(cmake_generator):].strip()
+        except (OSError, IOError):
+            pass
+
+        return None
+
+    def get_cached_generator_env(self):
+        """If any, return a mapping of environment associated with the cached generator.
+        """
+        generator_name = self.get_cached_generator_name()
+        if generator_name is not None:
+            return self.platform.get_generator(generator_name).env
+
+        return None
+
     def configure(self, clargs=(), generator_name=None,
-                  cmake_source_dir='.', cmake_install_dir='', cleanup=True):
+                  cmake_source_dir='.', cmake_install_dir='',
+                  languages=('C', 'CXX'), cleanup=True):
         """Calls cmake to generate the Makefile/VS Solution/XCode project.
 
         clargs: tuple
@@ -86,6 +135,11 @@ class CMaker(object):
             Relative directory to append
             to :const:`skbuild.constants.CMAKE_INSTALL_DIR`.
 
+        languages: tuple
+            List of languages required to configure the project and expected to
+            be supported by the compiler. The language identifier that can be specified
+            in the list corresponds to the one recognized by CMake.
+
         cleanup: bool
             If True, cleans up temporary folder used to test
             generators. Set to False for debugging to see CMake's
@@ -93,6 +147,8 @@ class CMaker(object):
 
         Return a mapping of the environment associated with the
         selected :class:`skbuild.platform_specifics.abstract.CMakeGenerator`.
+
+        Mapping of the environment can also be later retrieved using :meth:`.get_cached_generator_env`.
         """
 
         # if no provided default generator_name, check environment
@@ -107,7 +163,8 @@ class CMaker(object):
         # use the generator returned from the platform, with the current
         # generator_name as a suggestion
         generator = self.platform.get_best_generator(
-            generator_name, cmake_args=clargs, cleanup=cleanup)
+            generator_name, cmake_executable=self.cmake_executable, cmake_args=clargs,
+            languages=languages, cleanup=cleanup)
 
         if not os.path.exists(CMAKE_BUILD_DIR):
             os.makedirs(CMAKE_BUILD_DIR)
@@ -124,7 +181,7 @@ class CMaker(object):
 
         cmake_source_dir = os.path.abspath(cmake_source_dir)
         cmd = [
-            'cmake', cmake_source_dir, '-G', generator.name,
+            self.cmake_executable, cmake_source_dir, '-G', generator.name,
             ("-DCMAKE_INSTALL_PREFIX:PATH=" +
                 os.path.abspath(
                     os.path.join(CMAKE_INSTALL_DIR, cmake_install_dir))),
@@ -343,10 +400,10 @@ class CMaker(object):
 
         Indeed, we can not wait for the manifest, so we try to extract the
         information (install destination) from the CMake build files
-        ``*.cmake`` found in ``CMAKE_BUILD_DIR``.
+        ``*.cmake`` found in :const:`skbuild.constants.CMAKE_BUILD_DIR`.
 
-        It raises ``SKBuildError`` if it found install destination outside of
-        ``CMAKE_INSTALL_DIR``.
+        It raises :class:`skbuild.exceptions.SKBuildError` if it found install destination outside of
+        :const:`skbuild.constants.CMAKE_INSTALL_DIR`.
         """
 
         bad_installs = []
@@ -393,7 +450,7 @@ class CMaker(object):
                                 "Did you forget to run configure before "
                                 "make?").format(CMAKE_BUILD_DIR))
 
-        cmd = ["cmake", "--build", source_dir,
+        cmd = [self.cmake_executable, "--build", source_dir,
                "--target", "install", "--config", config, "--"]
         cmd.extend(clargs)
         cmd.extend(
@@ -425,7 +482,10 @@ class CMaker(object):
     def _parse_manifests(self):
         paths = \
             glob.glob(os.path.join(CMAKE_BUILD_DIR, "install_manifest*.txt"))
-        return [self._parse_manifest(path) for path in paths][0]
+        try:
+            return [self._parse_manifest(path) for path in paths][0]
+        except IndexError:
+            return []
 
     @staticmethod
     def _parse_manifest(install_manifest_path):
