@@ -4,16 +4,23 @@ from distutils and setuptools.
 
 from __future__ import print_function
 
+import copy
 import os
 import os.path
 import sys
 import argparse
+import json
+import platform
+import stat
 
 from contextlib import contextmanager
 from distutils.errors import (DistutilsArgError,
                               DistutilsError,
                               DistutilsGetoptError)
+from glob import glob
 from shutil import copyfile, copymode
+from packaging.requirements import Requirement
+from packaging.version import parse as parse_version
 
 # XXX If 'six' becomes a dependency, use 'six.StringIO' instead.
 try:
@@ -21,6 +28,11 @@ try:
 except ImportError:
     from io import StringIO
 
+try:
+    from shutil import which
+except ImportError:
+    from .compat import which
+
 from setuptools import setup as upstream_setup
 from setuptools.dist import Distribution as upstream_Distribution
 
@@ -28,10 +40,10 @@ from . import cmaker
 from .command import (build, build_py, clean,
                       install, install_lib, install_scripts,
                       bdist, bdist_wheel, egg_info,
-                      sdist, generate_source_manifest)
-from .constants import CMAKE_INSTALL_DIR
+                      sdist, generate_source_manifest, test)
+from .constants import CMAKE_INSTALL_DIR, CMAKE_SPEC_FILE
 from .exceptions import SKBuildError, SKBuildGeneratorNotFoundError
-from .utils import (mkdir_p, PythonModuleFinder, to_platform_path, to_unix_path)
+from .utils import (mkdir_p, parse_manifestin, PythonModuleFinder, to_platform_path, to_unix_path)
 
 
 def create_skbuild_argparser():
@@ -47,6 +59,10 @@ def create_skbuild_argparser():
     parser.add_argument(
         '-j', metavar='N', type=int, dest='jobs',
         help='allow N build jobs at once')
+    parser.add_argument(
+        '--cmake-executable', default=None, metavar='',
+        help='specify the path to the cmake executable'
+    )
     return parser
 
 
@@ -54,7 +70,7 @@ def parse_skbuild_args(args, cmake_args, build_tool_args):
     """
     Parse arguments in the scikit-build argument set. Convert specified
     arguments to proper format and append to cmake_args and build_tool_args.
-    Returns remaining arguments.
+    Returns the tuple ``(remaining arguments, cmake executable)``.
     """
     parser = create_skbuild_argparser()
     namespace, remaining_args = parser.parse_known_args(args)
@@ -69,13 +85,13 @@ def parse_skbuild_args(args, cmake_args, build_tool_args):
     if namespace.jobs is not None:
         build_tool_args.extend(['-j', str(namespace.jobs)])
 
-    return remaining_args
+    return remaining_args, namespace.cmake_executable
 
 
 def parse_args():
     """This function parses the command-line arguments ``sys.argv`` and returns
-    the tuple ``(setuptools_args, cmake_args, build_tool_args)`` where each
-    element corresponds to a set of arguments separated by ``--``."""
+    the tuple ``(setuptools_args, cmake_executable, cmake_args, build_tool_args)`` where each
+    ``*_args`` element corresponds to a set of arguments separated by ``--``."""
     dutils = []
     cmake = []
     make = []
@@ -94,9 +110,9 @@ def parse_args():
         else:
             argsets[i].append(arg)
 
-    dutils = parse_skbuild_args(dutils, cmake, make)
+    dutils, cmake_executable = parse_skbuild_args(dutils, cmake, make)
 
-    return dutils, cmake, make
+    return dutils, cmake_executable, cmake, make
 
 
 @contextmanager
@@ -118,7 +134,7 @@ def _parse_setuptools_arguments(setup_attrs):
 
     It returns the tuple
         ``(display_only, help_commands, commands,
-        hide_listing, force_cmake, skip_cmake)``
+        hide_listing, force_cmake, skip_cmake, plat_name)``
     where
     - display_only is a boolean indicating if an argument like '--help',
       '--help-commands' or '--author' was passed.
@@ -130,6 +146,8 @@ def _parse_setuptools_arguments(setup_attrs):
     - force_cmake a boolean indicating that CMake should always be executed.
     - skip_cmake is a boolean indicating if the execution of CMake should
       explicitly be skipped.
+    - plat_name is a string identifying the platform name to embed in generated
+      filenames. It defaults to ``distutils.util.get_platform()``.
 
     Otherwise it raises DistutilsArgError exception if there are
     any error on the command-line, and it raises DistutilsGetoptError
@@ -178,8 +196,20 @@ def _parse_setuptools_arguments(setup_attrs):
         if not hasattr(dist, 'skip_cmake'):
             dist.skip_cmake = False
 
+    plat_names = set()
+    for cmd in [dist.get_command_obj(command) for command in dist.commands]:
+        if getattr(cmd, 'plat_name', None) is not None:
+            plat_names.add(cmd.plat_name)
+    if not plat_names:
+        plat_names.add(None)
+    elif len(plat_names) > 1:
+        raise SKBuildError(
+            "--plat-name is ambiguous: %s" % ", ".join(plat_names))
+    plat_name = list(plat_names)[0]
+
     return (display_only, dist.help_commands, dist.commands,
-            dist.hide_listing, dist.force_cmake, dist.skip_cmake)
+            dist.hide_listing, dist.force_cmake, dist.skip_cmake,
+            plat_name)
 
 
 def _check_skbuild_parameters(skbuild_kw):
@@ -265,6 +295,7 @@ def _should_run_cmake(commands, cmake_with_sdist):
         "bdist_rpm",
         "bdist_wininst",
         "bdist_wheel",
+        "test",
     ]:
         if expected_command in commands:
             return True
@@ -273,13 +304,39 @@ def _should_run_cmake(commands, cmake_with_sdist):
     return False
 
 
+def _save_cmake_spec(args):
+    """Save the CMake spec to disk"""
+    # We use JSON here because readability is more important than performance
+    try:
+        os.makedirs(os.path.dirname(CMAKE_SPEC_FILE))
+    except OSError:
+        pass
+
+    with open(CMAKE_SPEC_FILE, 'w+') as fp:
+        json.dump(args, fp)
+
+
+def _load_cmake_spec():
+    """Load and return the CMake spec from disk"""
+    try:
+        with open(CMAKE_SPEC_FILE) as fp:
+            return json.load(fp)
+    except (OSError, IOError, ValueError):
+        return None
+
+
 # pylint:disable=too-many-locals, too-many-branches
 def setup(*args, **kw):  # noqa: C901
     """This function wraps setup() so that we can run cmake, make,
     CMake build, then proceed as usual with setuptools, appending the
     CMake-generated output as necessary.
+
+    The CMake project is re-configured only if needed. This is achieved by (1) retrieving the environment mapping
+    associated with the generator set in the ``CMakeCache.txt`` file, (2) saving the CMake configure arguments and
+    version in :const:`skbuild.constants.CMAKE_SPEC_FILE`: and (3) re-configuring only if either the generator or
+    the CMake specs change.
     """
-    sys.argv, cmake_args, make_args = parse_args()
+    sys.argv, cmake_executable, cmake_args, make_args = parse_args()
 
     # work around https://bugs.python.org/issue1011113
     # (patches provided, but no updates since 2014)
@@ -300,6 +357,7 @@ def setup(*args, **kw):  # noqa: C901
     cmdclass['generate_source_manifest'] = cmdclass.get(
         'generate_source_manifest',
         generate_source_manifest.generate_source_manifest)
+    cmdclass['test'] = cmdclass.get('test', test.test)
     kw['cmdclass'] = cmdclass
 
     # Extract setup keywords specific to scikit-build and remove them from kw.
@@ -310,7 +368,9 @@ def setup(*args, **kw):  # noqa: C901
         'cmake_args': [],
         'cmake_install_dir': '',
         'cmake_source_dir': '',
-        'cmake_with_sdist': False
+        'cmake_with_sdist': False,
+        'cmake_languages': ('C', 'CXX'),
+        'cmake_minimum_required_version': None
     }
     skbuild_kw = {param: kw.pop(param, parameters[param])
                   for param in parameters}
@@ -344,7 +404,8 @@ def setup(*args, **kw):  # noqa: C901
     commands = []
     try:
         (display_only, help_commands, commands,
-         hide_listing, force_cmake, skip_cmake) = \
+         hide_listing, force_cmake, skip_cmake,
+         plat_name) = \
             _parse_setuptools_arguments(kw)
     except (DistutilsArgError, DistutilsGetoptError):
         has_invalid_arguments = True
@@ -354,13 +415,12 @@ def setup(*args, **kw):  # noqa: C901
     if not has_cmakelists:
         print('skipping skbuild (no CMakeLists.txt found)')
 
-    skip_cmake = (skip_cmake
-                  or display_only
-                  or has_invalid_arguments
-                  or not _should_run_cmake(commands,
-                                           skbuild_kw["cmake_with_sdist"])
-                  or not has_cmakelists)
-    if skip_cmake and not force_cmake:
+    skip_skbuild = (display_only
+                    or has_invalid_arguments
+                    or not _should_run_cmake(commands,
+                                             skbuild_kw["cmake_with_sdist"])
+                    or not has_cmakelists)
+    if skip_skbuild and not force_cmake:
         if help_commands:
             # Prepend scikit-build help. Generate option descriptions using
             # argparse.
@@ -379,11 +439,11 @@ def setup(*args, **kw):  # noqa: C901
             print('')
         return upstream_setup(*args, **kw)
 
-    developer_mode = "develop" in commands
+    developer_mode = "develop" in commands or "test" in commands
 
     packages = kw.get('packages', [])
     package_dir = kw.get('package_dir', {})
-    package_data = kw.get('package_data', {}).copy()
+    package_data = copy.deepcopy(kw.get('package_data', {}))
 
     py_modules = kw.get('py_modules', [])
     new_py_modules = {py_module: False for py_module in py_modules}
@@ -396,17 +456,83 @@ def setup(*args, **kw):  # noqa: C901
         for parent_dir, file_list in kw.get('data_files', [])
     }
 
+    if sys.platform == 'darwin':
+
+        if plat_name is None:
+            # The following code is duplicated in bdist_wheel.finalize_options()
+            plat_name = "macosx-10.6-x86_64"
+
+        (_, version, machine) = plat_name.split('-')
+        if not cmaker.has_cmake_cache_arg(
+                cmake_args, 'CMAKE_OSX_DEPLOYMENT_TARGET'):
+            cmake_args.append(
+                '-DCMAKE_OSX_DEPLOYMENT_TARGET:STRING=%s' % version
+            )
+        if not cmaker.has_cmake_cache_arg(
+                cmake_args, 'CMAKE_OSX_ARCHITECTURES'):
+            cmake_args.append(
+                '-DCMAKE_OSX_ARCHITECTURES:STRING=%s' % machine
+            )
+
+    # Install cmake if listed in `setup_requires`
+    for package in kw.get('setup_requires', []):
+        if Requirement(package).name == 'cmake':
+            setup_requires = [package]
+            dist = upstream_Distribution({'setup_requires': setup_requires})
+            dist.fetch_build_eggs(setup_requires)
+
+            # Considering packages associated with "setup_requires" keyword are
+            # installed in .eggs subdirectory without honoring setuptools "console_scripts"
+            # entry_points and without settings the expected executable permissions, we are
+            # taking care of it below.
+            import cmake
+            for executable in ['cmake', 'cpack', 'ctest']:
+                executable = os.path.join(cmake.CMAKE_BIN_DIR, executable)
+                if platform.system().lower() == 'windows':
+                    executable += '.exe'
+                st = os.stat(executable)
+                permissions = (
+                        st.st_mode | stat.S_IXUSR | stat.S_IXGRP | stat.S_IXOTH
+                )
+                os.chmod(executable, permissions)
+            cmake_executable = os.path.join(cmake.CMAKE_BIN_DIR, 'cmake')
+            break
+
     # Since CMake arguments provided through the command line have more
     # weight and when CMake is given multiple times a argument, only the last
     # one is considered, let's prepend the one provided in the setup call.
     cmake_args = skbuild_kw['cmake_args'] + cmake_args
 
+    # Languages are used to determine a working generator
+    cmake_languages = skbuild_kw['cmake_languages']
+
     try:
-        cmkr = cmaker.CMaker()
-        env = cmkr.configure(cmake_args,
-                             cmake_source_dir=cmake_source_dir,
-                             cmake_install_dir=skbuild_kw['cmake_install_dir'])
-        cmkr.make(make_args, env=env)
+        if cmake_executable is None:
+            cmake_executable = 'cmake'
+        cmkr = cmaker.CMaker(cmake_executable)
+        if not skip_cmake:
+            cmake_minimum_required_version = skbuild_kw['cmake_minimum_required_version']
+            if cmake_minimum_required_version is not None:
+                if parse_version(cmkr.cmake_version) < parse_version(cmake_minimum_required_version):
+                    raise SKBuildError(
+                        "CMake version %s or higher is required. CMake version %s is being used" % (
+                            cmake_minimum_required_version, cmkr.cmake_version))
+            # Used to confirm that the cmake executable is the same
+            cmake_spec = {
+                'args': [which('cmake')] + cmake_args,
+                'version': cmkr.cmake_version
+            }
+
+            # skip the configure step for a cached build
+            env = cmkr.get_cached_generator_env()
+            if env is None or cmake_spec != _load_cmake_spec():
+                env = cmkr.configure(cmake_args,
+                                     cmake_source_dir=cmake_source_dir,
+                                     cmake_install_dir=skbuild_kw['cmake_install_dir'],
+                                     languages=cmake_languages
+                                     )
+                _save_cmake_spec(cmake_spec)
+            cmkr.make(make_args, env=env)
     except SKBuildGeneratorNotFoundError as ex:
         sys.exit(ex)
     except SKBuildError as ex:
@@ -419,16 +545,26 @@ def setup(*args, **kw):  # noqa: C901
     # If needed, set reasonable defaults for package_dir
     for package in packages:
         if package not in package_dir:
-            package_dir[package] = package.replace(".", os.path.sep)
+            package_dir[package] = package.replace(".", "/")
+            if '' in package_dir:
+                package_dir[package] = to_unix_path(os.path.join(package_dir[''], package_dir[package]))
 
     package_prefixes = _collect_package_prefixes(package_dir, packages)
 
-    _classify_files(cmkr.install(), package_data, package_prefixes,
-                    py_modules, new_py_modules,
-                    scripts, new_scripts,
-                    data_files,
-                    cmake_source_dir, skbuild_kw['cmake_install_dir'])
+    _classify_installed_files(cmkr.install(), package_data, package_prefixes,
+                              py_modules, new_py_modules,
+                              scripts, new_scripts,
+                              data_files,
+                              cmake_source_dir, skbuild_kw['cmake_install_dir'])
 
+    original_manifestin_data_files = []
+    if kw.get("include_package_data", False):
+        original_manifestin_data_files = parse_manifestin(os.path.join(os.getcwd(), "MANIFEST.in"))
+        for path in original_manifestin_data_files:
+            _classify_file(path, package_data, package_prefixes,
+                           py_modules, new_py_modules,
+                           scripts, new_scripts,
+                           data_files)
     if developer_mode:
         # Copy packages
         for package, package_file_list in package_data.items():
@@ -444,9 +580,15 @@ def setup(*args, **kw):  # noqa: C901
             if os.path.exists(cmake_file):
                 _copy_file(cmake_file, package_file, hide_listing)
     else:
-        _consolidate(cmake_source_dir,
-                     packages, package_dir, py_modules, package_data,
-                     hide_listing)
+        _consolidate_package_modules(
+            cmake_source_dir, packages, package_dir, py_modules, package_data, hide_listing)
+
+        original_package_data = kw.get('package_data', {}).copy()
+        _consolidate_package_data_files(original_package_data, package_prefixes, hide_listing)
+
+        for data_file in original_manifestin_data_files:
+            dest_data_file = os.path.join(CMAKE_INSTALL_DIR, data_file)
+            _copy_file(data_file, dest_data_file, hide_listing)
 
     kw['package_data'] = package_data
     kw['package_dir'] = {
@@ -522,11 +664,11 @@ def _collect_package_prefixes(package_dir, packages):
 
 
 # pylint:disable=too-many-arguments, too-many-branches
-def _classify_files(install_paths, package_data, package_prefixes,
-                    py_modules, new_py_modules,
-                    scripts, new_scripts,
-                    data_files,
-                    cmake_source_dir, cmake_install_dir):
+def _classify_installed_files(install_paths, package_data, package_prefixes,
+                              py_modules, new_py_modules,
+                              scripts, new_scripts,
+                              data_files,
+                              cmake_source_dir, cmake_install_dir):
     assert not os.path.isabs(cmake_source_dir)
     assert cmake_source_dir != "."
 
@@ -534,10 +676,6 @@ def _classify_files(install_paths, package_data, package_prefixes,
 
     install_root = os.path.join(os.getcwd(), CMAKE_INSTALL_DIR)
     for path in install_paths:
-        found_package = False
-        found_module = False
-        found_script = False
-
         # if this installed file is not within the project root, complain and
         # exit
         if not to_platform_path(path).startswith(CMAKE_INSTALL_DIR):
@@ -563,58 +701,73 @@ def _classify_files(install_paths, package_data, package_prefixes,
                 and not path.startswith(cmake_source_dir)):
             path = to_unix_path(os.path.join(cmake_source_dir, path))
 
-        # check to see if path is part of a package
-        for prefix, package in package_prefixes:
-            if path.startswith(prefix + "/"):
-                # peel off the package prefix
-                path = os.path.relpath(path, prefix)
+        _classify_file(path, package_data, package_prefixes,
+                       py_modules, new_py_modules,
+                       scripts, new_scripts,
+                       data_files)
 
-                package_file_list = package_data.get(package, [])
-                package_file_list.append(path)
-                package_data[package] = package_file_list
 
-                found_package = True
-                break
+def _classify_file(path, package_data, package_prefixes,
+                   py_modules, new_py_modules,
+                   scripts, new_scripts,
+                   data_files):
+    found_package = False
+    found_module = False
+    found_script = False
 
-        if found_package:
-            continue
-        # If control reaches this point, then this installed file is not part of
-        # a package.
+    path = to_unix_path(path)
 
-        # check if path is a module
-        for module in py_modules:
-            if path.replace("/", ".") == ".".join((module, "py")):
-                new_py_modules[module] = True
-                found_module = True
-                break
+    # check to see if path is part of a package
+    for prefix, package in package_prefixes:
+        if path.startswith(prefix + "/"):
+            # peel off the package prefix
+            path = to_unix_path(os.path.relpath(path, prefix))
 
-        if found_module:
-            continue
-        # If control reaches this point, then this installed file is not a
-        # module
+            package_file_list = package_data.get(package, [])
+            package_file_list.append(path)
+            package_data[package] = package_file_list
 
-        # if the file is a script, mark the corresponding script
-        for script in scripts:
-            if path == script:
-                new_scripts[script] = True
-                found_script = True
-                break
+            found_package = True
+            break
 
-        if found_script:
-            continue
-        # If control reaches this point, then this installed file is not a
-        # script
+    if found_package:
+        return
+    # If control reaches this point, then this installed file is not part of
+    # a package.
 
-        # If control reaches this point, then we have installed files that are
-        # not part of a package, not a module, nor a script.  Without any other
-        # information, we can only treat it as a generic data file.
-        parent_dir = os.path.dirname(path)
-        file_set = data_files.get(parent_dir)
-        if file_set is None:
-            file_set = set()
-            data_files[parent_dir] = file_set
-        file_set.add(os.path.join(CMAKE_INSTALL_DIR, path))
-        del parent_dir, file_set
+    # check if path is a module
+    for module in py_modules:
+        if path.replace("/", ".") == ".".join((module, "py")):
+            new_py_modules[module] = True
+            found_module = True
+            break
+
+    if found_module:
+        return
+    # If control reaches this point, then this installed file is not a
+    # module
+
+    # if the file is a script, mark the corresponding script
+    for script in scripts:
+        if path == script:
+            new_scripts[script] = True
+            found_script = True
+            break
+
+    if found_script:
+        return
+    # If control reaches this point, then this installed file is not a
+    # script
+
+    # If control reaches this point, then we have installed files that are
+    # not part of a package, not a module, nor a script.  Without any other
+    # information, we can only treat it as a generic data file.
+    parent_dir = os.path.dirname(path)
+    file_set = data_files.get(parent_dir)
+    if file_set is None:
+        file_set = set()
+        data_files[parent_dir] = file_set
+    file_set.add(os.path.join(CMAKE_INSTALL_DIR, path))
 
 
 def _copy_file(src_file, dest_file, hide_listing=True):
@@ -639,7 +792,7 @@ def _copy_file(src_file, dest_file, hide_listing=True):
     copymode(src_file, dest_file)
 
 
-def _consolidate(
+def _consolidate_package_modules(
         cmake_source_dir, packages, package_dir, py_modules, package_data,
         hide_listing
 ):
@@ -710,3 +863,35 @@ def _consolidate(
             package_data[package].append(stripped_module_file)
         except KeyError:
             package_data[package] = [stripped_module_file]
+
+
+def _consolidate_package_data_files(original_package_data, package_prefixes, hide_listing):
+    """This function copies package data files specified using the ``package_data`` keyword
+    into data::`.constants.CMAKE_INSTALL_DIR`.
+
+    ::
+
+        setup(...,
+            packages=['mypkg'],
+            package_dir={'mypkg': 'src/mypkg'},
+            package_data={'mypkg': ['data/*.dat']},
+            )
+
+    Considering that (1) the packages associated with modules located in both the source tree and
+    the CMake install tree are consolidated into the CMake install tree, and (2) the consolidated
+    package path set in the ``package_dir`` dictionary and later used by setuptools to package
+    (or install) modules and data files is data::`.constants.CMAKE_INSTALL_DIR`, copying the data files
+    is required to ensure setuptools can find them when it uses the package directory.
+    """
+    project_root = os.getcwd()
+    for prefix, package in package_prefixes:
+        if package not in original_package_data:
+            continue
+        raw_patterns = original_package_data[package]
+        for pattern in raw_patterns:
+            expanded_package_dir = os.path.join(project_root, prefix, pattern)
+            for src_data_file in glob(expanded_package_dir):
+                full_prefix_length = len(os.path.join(project_root, prefix)) + 1
+                data_file = src_data_file[full_prefix_length:]
+                dest_data_file = os.path.join(CMAKE_INSTALL_DIR, prefix, data_file)
+                _copy_file(src_data_file, dest_data_file, hide_listing)
