@@ -1,8 +1,10 @@
 # -*- coding: utf-8 -*-
 
 import _pytest.tmpdir
+import distutils
 import os
 import os.path
+import pkg_resources
 import py.path
 import re
 import six
@@ -12,7 +14,10 @@ import sys
 
 from contextlib import contextmanager
 
+from mock import patch
+from skbuild.compat import which  # noqa: F401
 from skbuild.utils import push_dir
+from skbuild.platform_specifics import get_platform
 
 
 SAMPLES_DIR = os.path.join(
@@ -45,6 +50,17 @@ def push_env(**kwargs):
         os.environ[saved_var] = saved_value
 
 
+@contextmanager
+def prepend_sys_path(paths):
+    """This context manager allows to prepend paths to ``sys.path`` and restore the
+    original list.
+    """
+    saved_paths = list(sys.path)
+    sys.path = paths + saved_paths
+    yield
+    sys.path = saved_paths
+
+
 def _tmpdir(basename):
     """This function returns a temporary directory similar to the one
     returned by the ``tmpdir`` pytest fixture.
@@ -170,27 +186,45 @@ def prepare_project(project, tmp_project_dir, force=False):
 
 
 @contextmanager
-def execute_setup_py(project_dir, setup_args):
+def execute_setup_py(project_dir, setup_args, disable_languages_test=False):
     """Context manager executing ``setup.py`` with the given arguments.
 
     It yields after changing the current working directory
     to ``project_dir``.
     """
 
-    with push_dir(str(project_dir)), \
-            push_argv(["setup.py"] + setup_args):
-        setup_code = None
+    # See https://stackoverflow.com/questions/9160227/dir-util-copy-tree-fails-after-shutil-rmtree
+    distutils.dir_util._path_created = {}
+
+    with push_dir(str(project_dir)), push_argv(["setup.py"] + setup_args), prepend_sys_path([str(project_dir)]):
+
+        # Restore master working set that is reset following call to "python setup.py test"
+        # See function "project_on_sys_path()" in setuptools.command.test
+        pkg_resources._initialize_master_working_set()
 
         with open("setup.py", "r") as fp:
             setup_code = compile(fp.read(), "setup.py", mode="exec")
 
-        if setup_code is not None:
-            six.exec_(setup_code)
+            if setup_code is not None:
+
+                if disable_languages_test:
+
+                    platform = get_platform()
+                    original_write_test_cmakelist = platform.write_test_cmakelist
+
+                    def write_test_cmakelist_no_languages(_self, _languages):
+                        original_write_test_cmakelist([])
+
+                    with patch.object(type(platform), 'write_test_cmakelist', new=write_test_cmakelist_no_languages):
+                        six.exec_(setup_code)
+
+                else:
+                    six.exec_(setup_code)
 
         yield
 
 
-def project_setup_py_test(project, setup_args, tmp_dir=None, verbose_git=True):
+def project_setup_py_test(project, setup_args, tmp_dir=None, verbose_git=True, disable_languages_test=False):
 
     def dec(fun):
 
@@ -203,7 +237,7 @@ def project_setup_py_test(project, setup_args, tmp_dir=None, verbose_git=True):
                 initialize_git_repo_and_commit(
                     wrapped.tmp_dir, verbose=wrapped.verbose_git)
 
-            with execute_setup_py(wrapped.tmp_dir, wrapped.setup_args):
+            with execute_setup_py(wrapped.tmp_dir, wrapped.setup_args, disable_languages_test=disable_languages_test):
                 result2 = fun(*iargs, **ikwargs)
 
             return wrapped.tmp_dir, result2
@@ -239,26 +273,3 @@ def get_cmakecache_variables(cmakecache):
                 variable_value = line.split("=")[1]
                 results[variable_name] = (variable_type, variable_value)
     return results
-
-
-def which(name, flags=os.X_OK):
-    """Analogue of unix 'which'. Borrowed from the Twisted project, see
-       their licence here: https://twistedmatrix.com/trac/browser/trunk/LICENSE
-
-       Copied from ``pytest_shutil.cmdline.which`` to allow testing on
-       conda-forge where ``pytest-shutil`` is not available.
-    """
-    result = []
-    exts = filter(None, os.environ.get('PATHEXT', '').split(os.pathsep))
-    path = os.environ.get('PATH', None)
-    if path is None:
-        return []
-    for p in os.environ.get('PATH', '').split(os.pathsep):
-        p = os.path.join(p, name)
-        if os.access(p, flags):
-            result.append(p)
-        for e in exts:
-            pext = p + e
-            if os.access(pext, flags):
-                result.append(pext)
-    return result
