@@ -11,7 +11,7 @@ its value.
 
 import pytest
 
-from subprocess import (check_call, CalledProcessError)
+from subprocess import (check_output, CalledProcessError)
 
 from skbuild.exceptions import SKBuildError
 from skbuild.platform_specifics import CMakeGenerator, get_platform
@@ -25,7 +25,7 @@ def test_cmakelists_with_fatalerror_fails(capfd):
 
     with push_dir():
 
-        @project_setup_py_test("fail-with-fatal-error-cmakelists", ["build"])
+        @project_setup_py_test("fail-with-fatal-error-cmakelists", ["build"], disable_languages_test=True)
         def should_fail():
             pass
 
@@ -48,7 +48,7 @@ def test_cmakelists_with_syntaxerror_fails(capfd):
 
     with push_dir():
 
-        @project_setup_py_test("fail-with-syntax-error-cmakelists", ["build"])
+        @project_setup_py_test("fail-with-syntax-error-cmakelists", ["build"], disable_languages_test=True)
         def should_fail():
             pass
 
@@ -98,19 +98,19 @@ def test_invalid_cmake(exception, mocker):
         CalledProcessError: CalledProcessError(['cmake', '--version'], 1)
     }
 
-    check_call_original = check_call
+    check_output_original = check_output
 
-    def check_call_mock(*args, **kwargs):
+    def check_output_mock(*args, **kwargs):
         if args[0] == ['cmake', '--version']:
             raise exceptions[exception]
-        check_call_original(*args, **kwargs)
+        check_output_original(*args, **kwargs)
 
-    mocker.patch('skbuild.cmaker.subprocess.check_call',
-                 new=check_call_mock)
+    mocker.patch('skbuild.cmaker.subprocess.check_output',
+                 new=check_output_mock)
 
     with push_dir():
 
-        @project_setup_py_test("hello", ["build"])
+        @project_setup_py_test("hello-no-language", ["build"], disable_languages_test=True)
         def should_fail():
             pass
 
@@ -137,7 +137,7 @@ def test_first_invalid_generator(mocker, capfd):
     mocker.patch('skbuild.cmaker.get_platform', return_value=platform)
 
     with push_dir(), push_env(CMAKE_GENERATOR=None):
-        @project_setup_py_test("hello", ["build"])
+        @project_setup_py_test("hello-no-language", ["build"])
         def run_build():
             pass
 
@@ -155,7 +155,7 @@ def test_invalid_generator(mocker, capfd):
     mocker.patch('skbuild.cmaker.get_platform', return_value=platform)
 
     with push_dir(), push_env(CMAKE_GENERATOR=None):
-        @project_setup_py_test("hello", ["build"])
+        @project_setup_py_test("hello-no-language", ["build"])
         def should_fail():
             pass
 
