@@ -12,9 +12,10 @@ import pytest
 import re
 import textwrap
 
-from skbuild.cmaker import (CMAKE_BUILD_DIR, CMaker)
+from skbuild.cmaker import CMaker, has_cmake_cache_arg
+from skbuild.constants import CMAKE_BUILD_DIR, CMAKE_INSTALL_DIR
 from skbuild.exceptions import SKBuildError
-from skbuild.utils import push_dir
+from skbuild.utils import push_dir, to_unix_path
 
 from . import _tmpdir, get_cmakecache_variables
 
@@ -29,6 +30,24 @@ def test_get_python_library():
     assert os.path.exists(python_library)
 
 
+def test_cmake_executable():
+    assert CMaker().cmake_executable == "cmake"
+
+
+def test_has_cmake_cache_arg():
+    cmake_args = ['-DFOO:STRING=42', '-DBAR', '-DCLIMBING:BOOL=ON']
+    assert has_cmake_cache_arg(cmake_args, "FOO", "42")
+    assert not has_cmake_cache_arg(cmake_args, "foo", "42")
+    assert not has_cmake_cache_arg(cmake_args, "FOO", "43")
+    assert not has_cmake_cache_arg(cmake_args, "BAR")
+    assert not has_cmake_cache_arg(cmake_args, "BA")
+    assert not has_cmake_cache_arg(cmake_args, "BAR", None)
+    assert not has_cmake_cache_arg(cmake_args, "BAR", "42")
+    assert has_cmake_cache_arg(cmake_args, "CLIMBING")
+    assert has_cmake_cache_arg(cmake_args, "CLIMBING", None)
+    assert has_cmake_cache_arg(cmake_args, "CLIMBING", "ON")
+
+
 def test_make_without_build_dir_fails():
     src_dir = _tmpdir('test_make_without_build_dir_fails')
     with push_dir(str(src_dir)), pytest.raises(SKBuildError) as excinfo:
@@ -78,13 +97,17 @@ def test_make(configure_with_cmake_source_dir, capfd):
         messages = ["Project has been installed"]
 
         if configure_with_cmake_source_dir:
-            messages += ["/SRC",
-                         "/BUILD/_skbuild/cmake-build",
-                         "/BUILD/_skbuild/cmake-install/./foo.txt"]
+            messages += [
+                "/SRC",
+                "/BUILD/{}".format(to_unix_path(CMAKE_BUILD_DIR)),
+                "/BUILD/{}/./foo.txt".format(to_unix_path(CMAKE_INSTALL_DIR))
+            ]
         else:
-            messages += ["/SRC",
-                         "/SRC/_skbuild/cmake-build",
-                         "/SRC/_skbuild/cmake-install/./foo.txt"]
+            messages += [
+                "/SRC",
+                "/SRC/{}".format(to_unix_path(CMAKE_BUILD_DIR)),
+                "/SRC/{}/./foo.txt".format(to_unix_path(CMAKE_INSTALL_DIR)),
+            ]
 
         out, _ = capfd.readouterr()
         for message in messages:
