@@ -13,11 +13,13 @@ import pprint
 import pytest
 
 from distutils.core import Distribution as distutils_Distribution
+from mock import patch
 from setuptools import Distribution as setuptool_Distribution
 
 from skbuild import setup as skbuild_setup
-from skbuild.cmaker import CMAKE_INSTALL_DIR
+from skbuild.constants import CMAKE_INSTALL_DIR, SKBUILD_DIR
 from skbuild.exceptions import SKBuildError
+from skbuild.platform_specifics import get_platform
 from skbuild.setuptools_wrap import strip_package
 from skbuild.utils import (push_dir, to_platform_path)
 
@@ -65,18 +67,26 @@ def test_distribution_is_pure(distribution_type, tmpdir):
         raise Exception(
             "Unknown distribution_type: {}".format(distribution_type))
 
-    with push_dir(str(tmpdir)), push_argv(["setup.py", "build"]):
-        distribution = skbuild_setup(
-            name="test",
-            version="0.0.1",
-            description="test object returned by setup function",
-            author="The scikit-build team",
-            license="MIT",
-            **skbuild_setup_kwargs
-        )
-        assert issubclass(distribution.__class__,
-                          (distutils_Distribution, setuptool_Distribution))
-        assert is_pure == distribution.is_pure()
+    platform = get_platform()
+    original_write_test_cmakelist = platform.write_test_cmakelist
+
+    def write_test_cmakelist_no_languages(_self, _languages):
+        original_write_test_cmakelist([])
+
+    with patch.object(type(platform), 'write_test_cmakelist', new=write_test_cmakelist_no_languages):
+
+        with push_dir(str(tmpdir)), push_argv(["setup.py", "build"]):
+            distribution = skbuild_setup(
+                name="test",
+                version="0.0.1",
+                description="test object returned by setup function",
+                author="The scikit-build team",
+                license="MIT",
+                **skbuild_setup_kwargs
+            )
+            assert issubclass(distribution.__class__,
+                              (distutils_Distribution, setuptool_Distribution))
+            assert is_pure == distribution.is_pure()
 
 
 @pytest.mark.parametrize("cmake_args", [
@@ -90,7 +100,7 @@ def test_cmake_args_keyword(cmake_args, capfd):
         """
         from skbuild import setup
         setup(
-            name="hello",
+            name="test_cmake_args_keyword",
             version="1.2.3",
             description="a minimal example package",
             author='The scikit-build team',
@@ -114,7 +124,7 @@ def test_cmake_args_keyword(cmake_args, capfd):
         """
     ))
 
-    with execute_setup_py(tmp_dir, ['build'] + cmake_args):
+    with execute_setup_py(tmp_dir, ['build'] + cmake_args, disable_languages_test=True):
         pass
 
     out, _ = capfd.readouterr()
@@ -184,7 +194,7 @@ def test_cmake_install_dir_keyword(
     tmp_dir.join('CMakeLists.txt').write(textwrap.dedent(
         """
         cmake_minimum_required(VERSION 3.5.0)
-        project(banana)
+        project(banana NONE)
         file(WRITE "${CMAKE_BINARY_DIR}/__init__.py" "")
         install(FILES "${CMAKE_BINARY_DIR}/__init__.py" DESTINATION ".")
         """
@@ -195,7 +205,7 @@ def test_cmake_install_dir_keyword(
     failed = False
     message = ""
     try:
-        with execute_setup_py(tmp_dir, ['build']):
+        with execute_setup_py(tmp_dir, ['build'], disable_languages_test=True):
             pass
     except SystemExit as e:
         # Error is not of type SKBuildError, it is expected to be
@@ -216,8 +226,8 @@ def test_cmake_install_dir_keyword(
                 "setup parameter 'cmake_install_dir' "
                 "is set to an absolute path.")
     else:
-        assert "copying {}".format(os.path.join(
-            *"_skbuild/cmake-install/banana/__init__.py".split("/"))) in out
+        init_py = to_platform_path("{}/banana/__init__.py".format(CMAKE_INSTALL_DIR))
+        assert "copying {}".format(init_py) in out
 
 
 @pytest.mark.parametrize("cmake_with_sdist", [True, False])
@@ -228,7 +238,7 @@ def test_cmake_with_sdist_keyword(cmake_with_sdist, capfd):
         """
         from skbuild import setup
         setup(
-            name="hello",
+            name="cmake_with_sdist_keyword",
             version="1.2.3",
             description="a minimal example package",
             author='The scikit-build team',
@@ -248,7 +258,7 @@ def test_cmake_with_sdist_keyword(cmake_with_sdist, capfd):
 
     initialize_git_repo_and_commit(tmp_dir)
 
-    with execute_setup_py(tmp_dir, ['sdist']):
+    with execute_setup_py(tmp_dir, ['sdist'], disable_languages_test=True):
         pass
 
     out, _ = capfd.readouterr()
@@ -259,6 +269,85 @@ def test_cmake_with_sdist_keyword(cmake_with_sdist, capfd):
         assert "Generating done" not in out
 
 
+def test_cmake_minimum_required_version_keyword():
+    tmp_dir = _tmpdir('cmake_minimum_required_version')
+
+    tmp_dir.join('setup.py').write(textwrap.dedent(
+        """
+        from skbuild import setup
+        setup(
+            name="cmake_with_sdist_keyword",
+            version="1.2.3",
+            description="a minimal example package",
+            author='The scikit-build team',
+            license="MIT",
+            cmake_minimum_required_version='99.98.97'
+        )
+        """
+    ))
+    tmp_dir.join('CMakeLists.txt').write(textwrap.dedent(
+        """
+        cmake_minimum_required(VERSION 3.5.0)
+        project(test NONE)
+        install(CODE "execute_process(
+          COMMAND \${CMAKE_COMMAND} -E sleep 0)")
+        """
+    ))
+
+    try:
+        with execute_setup_py(tmp_dir, ['build'], disable_languages_test=True):
+            pass
+    except SystemExit as e:
+        # Error is not of type SKBuildError, it is expected to be
+        # raised by distutils.core.setup
+        failed = isinstance(e.code, SKBuildError)
+        message = str(e)
+        assert failed
+        assert "CMake version 99.98.97 or higher is required." in message
+
+
+def test_setup_requires_keyword_include_cmake(mocker, capsys):
+
+    mock_setup = mocker.patch('skbuild.setuptools_wrap.upstream_setup')
+
+    tmp_dir = _tmpdir('setup_requires_keyword_include_cmake')
+
+    setup_requires = ['cmake>=3.10']
+
+    tmp_dir.join('setup.py').write(textwrap.dedent(
+        """
+        from skbuild import setup
+        setup(
+            name="cmake_with_sdist_keyword",
+            version="1.2.3",
+            description="a minimal example package",
+            author='The scikit-build team',
+            license="MIT",
+            setup_requires=[{setup_requires}]
+        )
+        """.format(setup_requires=",".join(["'%s'" % package for package in setup_requires]))
+    ))
+    tmp_dir.join('CMakeLists.txt').write(textwrap.dedent(
+        """
+        cmake_minimum_required(VERSION 3.5.0)
+        project(test NONE)
+        install(CODE "execute_process(
+          COMMAND \${CMAKE_COMMAND} -E sleep 0)")
+        """
+    ))
+
+    with execute_setup_py(tmp_dir, ['build'], disable_languages_test=True):
+        assert mock_setup.call_count == 1
+        setup_kw = mock_setup.call_args[1]
+        assert setup_kw['setup_requires'] == setup_requires
+
+        import cmake
+
+        out, _ = capsys.readouterr()
+        if "Searching for cmake>=3.10" in out:
+            assert cmake.__file__.startswith(str(tmp_dir))
+
+
 @pytest.mark.parametrize("distribution_type", ('pure', 'skbuild'))
 def test_script_keyword(distribution_type, capsys):
 
@@ -305,7 +394,7 @@ def test_script_keyword(distribution_type, capsys):
         tmp_dir.join('CMakeLists.txt').write(textwrap.dedent(
             """
             cmake_minimum_required(VERSION 3.5.0)
-            project(foo)
+            project(foo NONE)
             file(WRITE "${CMAKE_BINARY_DIR}/foo.py" "# foo.py")
             file(WRITE "${CMAKE_BINARY_DIR}/bar.py" "# bar.py")
             install(
@@ -318,8 +407,8 @@ def test_script_keyword(distribution_type, capsys):
         ))
 
         messages = [
-            "copying _skbuild/cmake-install/{}.py -> "
-            "_skbuild/setuptools/scripts-".format(module)
+            "copying {}/{}.py -> "
+            "{}/setuptools/scripts-".format(CMAKE_INSTALL_DIR, module, SKBUILD_DIR)
             for module in ['foo', 'bar']]
 
     elif distribution_type == 'pure':
@@ -328,10 +417,10 @@ def test_script_keyword(distribution_type, capsys):
 
         messages = [
             "copying {}.py -> "
-            "_skbuild/setuptools/scripts-".format(module)
+            "{}/setuptools/scripts-".format(module, SKBUILD_DIR)
             for module in ['foo', 'bar']]
 
-    with execute_setup_py(tmp_dir, ['build']):
+    with execute_setup_py(tmp_dir, ['build'], disable_languages_test=True):
         pass
 
     out, _ = capsys.readouterr()
@@ -385,7 +474,7 @@ def test_py_modules_keyword(distribution_type, capsys):
         tmp_dir.join('CMakeLists.txt').write(textwrap.dedent(
             """
             cmake_minimum_required(VERSION 3.5.0)
-            project(foobar)
+            project(foobar NONE)
             file(WRITE "${CMAKE_BINARY_DIR}/foo.py" "# foo.py")
             file(WRITE "${CMAKE_BINARY_DIR}/bar.py" "# bar.py")
             install(
@@ -398,8 +487,8 @@ def test_py_modules_keyword(distribution_type, capsys):
         ))
 
         messages = [
-            "copying _skbuild/cmake-install/{}.py -> "
-            "_skbuild/setuptools/lib".format(module)
+            "copying {}/{}.py -> "
+            "{}/setuptools/lib".format(CMAKE_INSTALL_DIR, module, SKBUILD_DIR)
             for module in ['foo', 'bar']]
 
     elif distribution_type == 'pure':
@@ -408,10 +497,10 @@ def test_py_modules_keyword(distribution_type, capsys):
 
         messages = [
             "copying {}.py -> "
-            "_skbuild/setuptools/lib".format(module)
+            "{}/setuptools/lib".format(module, SKBUILD_DIR)
             for module in ['foo', 'bar']]
 
-    with execute_setup_py(tmp_dir, ['build']):
+    with execute_setup_py(tmp_dir, ['build'], disable_languages_test=True):
         pass
 
     out, _ = capsys.readouterr()
@@ -479,18 +568,18 @@ def test_setup_inputs(
     #             pure.py
     #
     #             data/
-    #                 pure.dat                   *NO TEST*
+    #                 pure.dat
     #
     #     [<cmake_src_dir>/]
     #
     #         hybrid/
     #             CMakeLists.txt
     #             __init__.py
-    #             hybrid_pure.dat                *NO TEST*
+    #             hybrid_pure.dat
     #             hybrid_pure.py
     #
     #             data/
-    #                 hybrid_data_pure.dat       *NO TEST*
+    #                 hybrid_data_pure.dat
     #
     #             hybrid_2/
     #                 __init__.py
@@ -517,11 +606,13 @@ def test_setup_inputs(
     #         cmake.py                   # CMake
     #
     #     hybrid/
-    #         hybrid_cmake.dat           # CMake                  *NO TEST*
+    #         hybrid_cmake.dat           # CMake
     #         hybrid_cmake.py            # CMake
+    #         hybrid_pure.dat            # Setuptools
+    #         hybrid_pure.py             # Setuptools
     #
     #         data/
-    #             hybrid_data_pure.dat   # CMake or Setuptools    *NO TEST*
+    #             hybrid_data_pure.dat   # CMake or Setuptools
     #             hybrid_data_cmake.dat  # CMake                  *NO TEST*
     #
     #         hybrid_2/
@@ -541,7 +632,7 @@ def test_setup_inputs(
     #         pure.py                    # Setuptools
     #
     #         data/
-    #             pure.dat               # Setuptools    *NO TEST*
+    #             pure.dat               # Setuptools
 
     tmp_dir.join('setup.py').write(textwrap.dedent(
         """
@@ -555,6 +646,7 @@ def test_setup_inputs(
             author='The scikit-build team',
             license="MIT",
             cmake_source_dir='{cmake_source_dir}',
+            cmake_install_dir='{cmake_install_dir}',
             # Arbitrary order of packages
             packages=[
         {p_off}    'pure',
@@ -567,9 +659,10 @@ def test_setup_inputs(
         {pm_off}   '{package_base}pureModule',
         {cm_off}   '{package_base}cmakeModule',
             ],
-            #package_data=[
-            #    '': ['*.dat']
-            #]
+            package_data={{
+        {p_off}        'pure': ['data/pure.dat'],
+        {h_off}        'hybrid': ['hybrid_pure.dat', 'data/hybrid_data_pure.dat'],
+            }},
             # Arbitrary order of package_dir
             package_dir = {{
         {p_off}    'hybrid.hybrid_2_pure': '{package_base}hybrid/hybrid_2_pure',
@@ -581,6 +674,7 @@ def test_setup_inputs(
         )
         """.format(
             cmake_source_dir=cmake_source_dir,
+            cmake_install_dir=package_base,
             package_base=package_base_dir,
             c_off='' if has_cmake_package else '#',
             cm_off='' if has_cmake_module else '#',
@@ -595,7 +689,7 @@ def test_setup_inputs(
     src_dir.join('CMakeLists.txt').write(textwrap.dedent(
         """
         cmake_minimum_required(VERSION 3.5.0)
-        project(hybrid)
+        project(hybrid NONE)
         set(build_dir ${{CMAKE_BINARY_DIR}})
 
         {c_off} file(WRITE ${{build_dir}}/__init__.py "")
@@ -612,11 +706,21 @@ def test_setup_inputs(
         {cm_off}     FILES ${{build_dir}}/cmakeModule.py
         {cm_off}     DESTINATION .)
 
+        {h_off} file(WRITE ${{build_dir}}/hybrid_cmake.dat "")
+        {h_off} install(
+        {h_off}     FILES ${{build_dir}}/hybrid_cmake.dat
+        {h_off}     DESTINATION hybrid)
+
         {h_off} file(WRITE ${{build_dir}}/hybrid_cmake.py "")
         {h_off} install(
         {h_off}     FILES ${{build_dir}}/hybrid_cmake.py
         {h_off}     DESTINATION hybrid)
 
+        {h_off} file(WRITE ${{build_dir}}/hybrid_data_cmake.dat "")
+        {h_off} install(
+        {h_off}     FILES ${{build_dir}}/hybrid_data_cmake.dat
+        {h_off}     DESTINATION hybrid/data)
+
         {h_off} file(WRITE ${{build_dir}}/hybrid_2_cmake.py "")
         {h_off} install(
         {h_off}     FILES ${{build_dir}}/hybrid_2_cmake.py
@@ -655,7 +759,7 @@ def test_setup_inputs(
     # Commented paths are the one expected to be installed by CMake. For
     # this reason, corresponding files should NOT be created in the source
     # tree.
-    for (type, path) in select_paths([
+    for (_type, path) in select_paths([
         # ('c', 'cmake/__init__.py'),
         # ('c', 'cmake/cmake.py'),
 
@@ -685,9 +789,9 @@ def test_setup_inputs(
 
         ('p', 'pure/data/pure.dat'),
     ]):
-        assert type in ['p', 'pm', 'h']
+        assert _type in ['p', 'pm', 'h']
         root = (package_base
-                if (type == 'p' or type == 'pm')
+                if (_type == 'p' or _type == 'pm')
                 else cmake_source_dir)
         tmp_dir.ensure(os.path.join(root, path))
 
@@ -704,7 +808,7 @@ def test_setup_inputs(
             "{}\n".format(desc, pprint.pformat(
                 setup_kw.get(desc, {}) if value is None else value, indent=2)))
 
-    with execute_setup_py(tmp_dir, ['build']):
+    with execute_setup_py(tmp_dir, ['build'], disable_languages_test=True):
 
         assert mock_setup.call_count == 1
         setup_kw = mock_setup.call_args[1]
@@ -743,12 +847,12 @@ def test_setup_inputs(
         if has_hybrid_package:
             expected_package_data['hybrid'] = [
                 '__init__.py',
-                # 'hybrid_cmake.dat',
+                'hybrid_cmake.dat',
                 'hybrid_cmake.py',
-                # 'hybrid_pure.dat'
+                'hybrid_pure.dat',
                 'hybrid_pure.py',
-                # 'data/hybrid_data_cmake.dat',
-                # 'data/hybrid_data_pure.dat',
+                'data/hybrid_data_cmake.dat',
+                'data/hybrid_data_pure.dat',
             ]
             expected_package_data['hybrid.hybrid_2'] = [
                 '__init__.py',
@@ -766,7 +870,7 @@ def test_setup_inputs(
             expected_package_data['pure'] = [
                 '__init__.py',
                 'pure.py',
-                # 'data/pure.dat',
+                'data/pure.dat',
             ]
 
         if has_cmake_module or has_pure_module:
@@ -882,7 +986,7 @@ def test_cmake_install_into_pure_package(with_cmake_source_dir, capsys):
     cmake_src_dir.join('CMakeLists.txt').write(textwrap.dedent(
         """
         cmake_minimum_required(VERSION 3.5.0)
-        project(test)
+        project(test NONE)
         file(WRITE "${CMAKE_BINARY_DIR}/apple.py" "# apple.py")
         file(WRITE "${CMAKE_BINARY_DIR}/banana.py" "# banana.py")
         install(
@@ -904,12 +1008,12 @@ def test_cmake_install_into_pure_package(with_cmake_source_dir, capsys):
 
     tmp_dir.ensure('fruits/__init__.py')
 
-    with execute_setup_py(tmp_dir, ['build']):
+    with execute_setup_py(tmp_dir, ['build'], disable_languages_test=True):
         pass
 
     messages = [
-        "copying _skbuild/cmake-install/{} -> "
-        "_skbuild/setuptools/lib".format(module)
+        "copying {}/{} -> "
+        "{}/setuptools/lib".format(CMAKE_INSTALL_DIR, module, SKBUILD_DIR)
         for module in [
             'fruits/__init__.py',
             'fruits/apple.py',
